// EOControlExtensions.h
// Copyright (c) 1997, Apple Computer, Inc. All rights reserved. 

#import <EOControl/EOControl.h>

@interface NSObject (JavaBridge)

- (void)willRead;
- (NSException *)validateValueInArray:(NSMutableArray *)valueArray forKey:(NSString *)key;

@end

@interface EOClassDescription (JavaBridge)

+ (NSException *)validationExceptionWithReason:(NSString *)reason;
+ (NSException *)aggregateExceptionWithExceptions:(NSArray *)subexceptions;
+ (NSException *)exceptionAddingEntriesToUserInfo:(NSException *)exception additions:(NSDictionary *)additions;

- (id)createInstanceWithEditingContext:(EOEditingContext *)editingContext globalID:(EOGlobalID *)globalID;
- (NSException *)validateValueInArray:(NSMutableArray *)valueArray forKey:(NSString *)key;

@end

@interface EOEditingContext (JavaBridge)

- (void)refault;
- (void)refetch;
- (void)undo;
- (void)redo;

@end

@interface EOFaultHandler (JavaBridge)

+ (void)makeObjectIntoFault:(id)object withHandler:(EOFaultHandler *)handler;
+ (BOOL)isFault:(id)object;
+ (void)clearFault:(id)fault;
+ (EOFaultHandler *)handlerForFault:(id)fault;

@end

@interface EOFetchSpecification (JavaBridge)

// We have to use initWithName... to avoid clashing with
// - initWithEntityName:(NSString *)entityName qualifier:(EOQualifier *)qualifier
//     sortOrderings:(NSArray *)sortOrderings usesDistinct:(BOOL)usesDistinct
//     isDeep:(BOOL)isDeep hints:(NSDictionary *)hints;
// since we can't remap the name of a constructor. Yuck.
- (id)initWithName:(NSString *)entityName qualifier:(EOQualifier *)qualifier sortOrderings:(NSArray *)sortOrderings;

@end

@interface EOQualifier (JavaBridge)

+ (NSArray *)filteredArray:(NSArray *)array withQualifier:(EOQualifier *)qualifier;
+ (void)filterArray:(NSMutableArray *)array withQualifier:(EOQualifier *)qualifier;

- (id)initWithQualifierFormat:(NSString *)format arguments:(NSArray *)args;

@end

@interface EOSortOrdering (JavaBridge)

+ (NSArray *)sortedArray:(NSArray *)array usingKeyOrderArray:(NSArray *)orderArray;
+ (void)sortArray:(NSMutableArray *)array usingKeyOrderArray:(NSArray *)orderArray;

@end

@interface EOKeyGlobalID (JavaBridge)

+ (id)globalIDWithEntityName:(NSString *)entityName keyArray:(NSArray *)keyArray;

@end

// Delegate protocols

@protocol EOClassDescriptionClassDelegates

- (BOOL)shouldPropagateDeleteForObject:(id)object inEditingContext:(EOEditingContext *)ec forRelationshipKey:(NSString *)key;

@end

@protocol EOEditingContextDelegates

- (BOOL)editingContext:(EOEditingContext *)editingContext shouldPresentException:(NSException *)exception;
- (BOOL)editingContextShouldValidateChanges:(EOEditingContext *)editingContext;
- (void)editingContextWillSaveChanges:(EOEditingContext *)editingContext;
- (BOOL)editingContext:(EOEditingContext *)editingContext shouldInvalidateObject:(id)object globalID:(EOGlobalID *)gid;
- (BOOL)editingContextShouldUndoUserActionsAfterFailure:(EOEditingContext *)context;
- (NSArray *)editingContext:(EOEditingContext *)editingContext shouldFetchObjectsDescribedByFetchSpecification:(EOFetchSpecification *)fetchSpecification;

@end

@protocol EOEditors

- (BOOL)editorHasChangesForEditingContext:(EOEditingContext *)editingContext;
- (void)editingContextWillSaveChanges:(EOEditingContext *)editingContext;

@end

@protocol EOMessageHandlers

- (void)editingContext:(EOEditingContext *)editingContext presentErrorMessage:(NSString *)message;
- (BOOL)editingContext:(EOEditingContext *)editingContext shouldContinueFetchingWithCurrentObjectCount:(unsigned)count originalLimit:(unsigned)limit objectStore:(EOObjectStore *)objectStore;

@end

@protocol EOKeyValueArchiverAwaking

- (void)finishInitializationWithKeyValueUnarchiver:(EOKeyValueUnarchiver *)unarchiver;
- (void)awakeFromKeyValueUnarchiver:(EOKeyValueUnarchiver *)unarchiver;

@end

@protocol EOKeyValueArchiverDelegates

- (id)archiver:(EOKeyValueArchiver *)archiver referenceToEncodeForObject:(id)object;

@end

@protocol EOKeyValueUnarchiverDelegates

- (id)unarchiver:(EOKeyValueUnarchiver *)archiver objectForReference:(id)object;

@end

@protocol EnterpriseObject

// EOInitializiation
- (id)initWithEditingContext:(EOEditingContext *)ec classDescription:(EOClassDescription *)classDesc globalID:(EOGlobalID *)globalID;

// EOClassDescriptionPrimitives
- (EOClassDescription *)classDescription;
- (NSString *)entityName;
- (NSArray *)attributeKeys;
- (NSArray *)toOneRelationshipKeys;
- (NSArray *)toManyRelationshipKeys;
- (NSString *)inverseForRelationshipKey:(NSString *)relationshipKey;
- (EODeleteRule)deleteRuleForRelationshipKey:(NSString *)relationshipKey;
- (BOOL)ownsDestinationObjectsForRelationshipKey:(NSString *)relationshipKey;
- (EOClassDescription *)classDescriptionForDestinationKey:(NSString *)detailKey;
- (NSString *)userPresentableDescription;
- (NSException *)validateForSave;
- (NSException *)validateForDelete;
- (void)awakeFromInsertionInEditingContext:(EOEditingContext *)editingContext;
- (void)awakeFromFetchInEditingContext:(EOEditingContext *)editingContext;

// EOClassDescriptionExtras
- (NSDictionary *)snapshot;
- (void)updateFromSnapshot:(NSDictionary *)snapshot;
- (BOOL)isToManyKey:(NSString *)key;
- (NSException *)validateForInsert;
- (NSException *)validateForUpdate;
- (NSArray *)allPropertyKeys;
- (void)clearProperties;
- (void)propagateDeleteWithEditingContext:(EOEditingContext *)editingContext;
- (NSString *)eoShallowDescription;
- (NSString *)eoDescription;

// EOKeyRelationshipManipulation
- (void)addObject:object toPropertyWithKey:(NSString *)key;
- (void)removeObject:object fromPropertyWithKey:(NSString *)key;
- (void)addObject:object toBothSidesOfRelationshipWithKey:(NSString *)key;
- (void)removeObject:object fromBothSidesOfRelationshipWithKey:(NSString *)key;

// EOEditingContext
- (EOEditingContext *)editingContext;

// EOKeyValueCodingPrimitives
- (id)valueForKey:(NSString *)key;
- (void)takeValue:(id)value forKey:(NSString *)key;
- (void)takeStoredValue:(id)value forKey:(NSString *)key;
//+ (BOOL)accessInstanceVariablesDirectly;
//+ (BOOL)useStoredAccessor;

// EOKVCPAdditions
- (id)valueForKeyPath:(NSString *)key;
- (void)takeValue:value forKeyPath:(NSString *)key;
- (void)takeStoredValue:value forKeyPath:(NSString *)key;
- (NSDictionary *)valuesForKeys:(NSArray *)keys;
- (void)takeValuesFromDictionary:(NSDictionary *)dictionary;

// EOKeyValueCodingException
- (id)handleQueryWithUnboundKey:(NSString *)key;
- (void)handleTakeValue:(id)value forUnboundKey:(NSString *)key;
- (void)unableToSetNilForKey:(NSString *)key;

// EOKeyValueCodingCacheControl
//+ (void)flushClassKeyBindings;
//+ (void)flushAllKeyBindings;

// EOObserver
- (void)willChange;

// EOControlExtensions JavaBridge
- (void)willRead;
- (NSException *)validateValueInArray:(NSMutableArray *)valueArray forKey:(NSString *)key;

@end

@interface EOGenericRecord (JavaBridge) <NSCoding, EnterpriseObject>
@end

@interface NSAttributedString (JavaBridge)

- (id)initUsingString:(NSString *)string attributes:(NSDictionary *)attributes;

@end

@interface NSFormatter (JavaBridge)

- (BOOL)getObjectValueInArray:(NSMutableArray *)valueArray forString:(NSString *)string errorDescriptionArray:(NSMutableArray *)errorArray;

@end
