// EODetailDataSource.h
// Copyright (c) 1995, NeXT Software, Inc. All rights reserved.
//
// The EODetailDataSource provides values extracted from a named property
// of a master object.  E.g. the master object could be a Department and the
// the detailKey could be "employees".  In this case the dataSource would
// use [department valueForKey:@"employees"] to get the set of objects
// to return from fetchObjects.  Values inserted or deleted in the dataSource
// would, in turn, cause insertions or deletion in the employees array of
// the department object.
//
#import <EOControl/EODataSource.h>

@interface EODetailDataSource : EODataSource <NSCoding>
{
    EODataSource *_masterDataSource;
    id _masterObject;
    NSString *_detailKey;
    NSString *_masterClassDescriptionName;
}
- initWithMasterClassDescription:(EOClassDescription *)masterClassDescription
               detailKey:(NSString *)detailKey;
    // primitive initializer

- initWithMasterDataSource:(EODataSource *)master detailKey:(NSString *)detailKey;
    // extracts editingContext as master class description from master
    // dataSource and calls primitive initializer

- (EODataSource *)masterDataSource;

- (EOClassDescription *)masterClassDescription;
- (void)setMasterClassDescription:(EOClassDescription *)classDescription;

- (void)qualifyWithRelationshipKey:(NSString *)key ofObject:masterObject;
- (NSString *)detailKey;
- (id)masterObject;

- (EOEditingContext *)editingContext;
    // returns the editing context for the master object (nil if none).

@end
