// EODeallocNotification.h
// Copyright (c) 1995, NeXT Software, Inc. All rights reserved.
//
#import <Foundation/Foundation.h>


@protocol EODeallocNotification
- (void)objectDeallocated:(id)object;
    // Called when "object" is deallocated.  Receiver should not send
    // messages to object or otherwise depend on its state because
    // it will have been partially deallocated.
@end

@interface NSObject (EODeallocNotification)
+ (void)registerObjectForDeallocNotification:(id <EODeallocNotification>)object;
    // The given object will be called with objectDealocated: whenever the
    // receiving class or its subclasses are dealloced.  This method
    // can be called for the same class with the same object without ill effect.
    // IMPROPER USE OF DEALLOC NOTIFICATION CAN GREATLY DAMAGE PERFORMANCE.
    // For example, one should never hack NSObject, NSString, or any class that
    // for which instances are often created and destroyed.
@end
