// _EOSetEnviron.h
// Incredibly spooky code to let apps access and set the environment.
// Special spookyness is required to allow prebinding of adaptors on
// Mach (which would otherwise fail because of an unresolved reference
// to the "environ" symbol declared in crt0.o
//
// To use, the adaptor framework .m file that needs to call set environ
// must include this file.  In addition, another otherwise empty .m
// file must include the following:
//   #define EOSETENVIRON_INCLUDE_IMPLEMENTATION
//   #import <EOControl/_EOSetEnviron.h>
//
// _EOInitEnviron() must be called before the environment is examined by 
// any code linked into the adaptor framework.

#ifdef SolarisOpenStep
#import <stdlib.h>
#endif

#if defined(__MACH__)
__private_extern__ void _EOSetEnviron(const char *key, const char *value);
__private_extern__ void _EOInitEnviron(void);
#else
void _EOSetEnviron(const char *key, const char *value);
void _EOInitEnviron(void);
#endif


#ifdef EOSETENVIRON_INCLUDE_IMPLEMENTATION

#if defined(__MACH__)

#import <libc.h>

__private_extern__ char **environ = NULL;

__private_extern__ void _EOInitEnviron(void)
{
    extern char ***_NSGetEnviron(void);
    if (!environ) {
        environ = *_NSGetEnviron();
    }
}

static int _EOPutEnv(char *newval)
{
    // Code below from the SATAN NeXTanswer
    static int firstTime = 1;
    char **ep;
    char *cp;
    int esiz;
    char *np;

    _EOInitEnviron();
    
    if (!(np = strchr(newval, '=')))
	return 1;
    *np = '\0';

    /* look it up */
    for (ep=environ ; *ep ; ep++) {
	/* this should always be true... */
	if ( (cp = strchr(*ep, '=')) ) {
	    *cp = '\0';
	    if (!strcmp(*ep, newval)) {
		/* got it! */
		*cp = '=';
		break;
	    }
	    *cp = '=';
	} else {
	    *np = '=';
	    return 1;
	}
    }

    *np = '=';
    if (*ep) {
	/* the string was already there: just replace it with the new one */
	*ep = newval;
	return 0;
    }

    /* expand environ by one */
    for (esiz=2, ep=environ ; *ep ; ep++)
	esiz++;
    if (firstTime) {
	char **epp;
	char **newenv;
	if (!(newenv = malloc(esiz * sizeof(char *))))
	    return 1;
   
	for (ep=environ, epp=newenv ; *ep ;)
	    *epp++ = *ep++;
	*epp++ = newval;
	*epp = (char *) 0;
	environ = newenv;
    } else {
	if (!(environ = realloc(environ, esiz * sizeof(char *))))
	    return 1;
	environ[esiz - 2] = newval;
	environ[esiz - 1] = (char *) 0;
	firstTime = 0;
    }

    return 0;
}

static char *
nvmatch(s1, s2)
register char *s1, *s2;
{

        while (*s1 == *s2++)
                if (*s1++ == '=')
                        return(s2);
        if (*s1 == '\0' && *(s2-1) == '=')
                return(s2);
        return(NULL);
}

__private_extern__ char *getenv(const char *name)
{
        register char **p;
        register char *v;

        p = environ;
        while (*p != NULL)
                if ((v = nvmatch(name, *p++)) != NULL)
                        return(v);
        return(NULL);
}

__private_extern__ void _EOSetEnviron(const char *key, const char *val)
{
    char *str = malloc(strlen(key) + strlen(val) + 2);
    _EOPutEnv(strcat(strcat(strcpy(str, key), "="), val));
}

#elif defined(SolarisOpenStep)
// We may be able to use this version for all non-Mach platforms (not just SolarisOpenStep)
extern "C" {
#import <stdlib.h>
#import <string.h>
extern int putenv(const char *foo);
}

void _EOInitEnviron(void)
{
}

void _EOSetEnviron(const char *key, const char *val)
{
    char *str = (char *)malloc(strlen(key) + strlen(val) + 2);
    putenv(strcat(strcat(strcpy(str, key), "="), val));
}
#else
#import <stdlib.h>
#import <string.h>

void _EOInitEnviron(void)
{
}

void _EOSetEnviron(const char *key, const char *val)
{
    char *str = (char *)malloc(strlen(key) + strlen(val) + 2);
    putenv(strcat(strcat(strcpy(str, key), "="), val));
}
#endif  // #if defined(MACH)

#endif  // EOSETENVIRON_INCLUDE_IMPLEMENTAION

