// private.h
// Copyright (c) 1995, NeXT Software, Inc. All rights reserved.
//
// Internal interfaces shared across control layer.  Included by many
// Control Layer .m files.

#ifdef SolarisOpenStep
#import <EOControl/SolarisEOF.h>
#endif

#import <EOControl/EOFoundationExtras.h>
#import <EOControl/_EOMutableKnownKeyDictionary.h>
#import <EOControl/_EOInternallyRefCountedObject.h>
#import <EOControl/_EOCheapCopyArray.h>
#import <EOControl/EOQualifier.h>
#import <EOControl/EOProxyDataSource.h>
#import <EOControl/EOClassDescription.h>
#import <EOControl/EOKeyGlobalID.h>

#import <Foundation/NSPrivate.h>

#ifndef SolarisOpenStep
#import <Foundation/NSDebug.h>
#endif

#import <EOControl/EODefines.h>

// Since foundation doesn't export _NSDBGEnabled, we can't easily use it.  If
// you want to be able to do 'oh' on the EOF classes which keep their own
// refcounts, set this to YES.
#define _EODBGEnabled NSDebugEnabled

// From 4.2 to Yellow, Foundation changed the names of the following enumerated types
// So we #ifdef them here.
#ifdef YELLOW
#define NSObjectInternalRefIncrementedAllocationEvent NSObjectInternalRefIncrementedEvent
#define NSObjectInternalRefDecrementedAllocationEvent NSObjectInternalRefDecrementedEvent
#endif

// Every class which does its own refcounting should call
// _EOInitGlobalRetainLock in its initialize method, and use
// _EOGlobalRetainLock to protect retain/release.
EOCONTROL_EXTERN void _EOInitGlobalRetainLock (void);
EOCONTROL_EXTERN id <NSLocking> _EOGlobalRetainLock;


EOCONTROL_EXTERN void _EORequestConcreteImplementation(id self, SEL _cmd, Class absClass);


@interface NSDictionary (PointerBasedManipulation)

+ (NSDictionary *)dictionaryWithNullValuesForKeys:(NSArray *)keys;

- (NSDictionary *)dictionaryExcludingEquivalentValuesInDictionary:(NSDictionary *)dict;
    // returns a dictionary containing all entries in self that have different object
    // values in "dict".

@end


@interface EOClassDescription (EONonCPExtensions)
// default implementation call non-all version.
// EOEntityClassDescription overrides with model-savvy implementation.
- (NSArray *)allAttributeKeys;
     // like attributeKeys, but returns non-class properties as well
- (NSArray *)allToOneRelationshipKeys;
    // like toOneRelationshipKeys, but returns non-class properties as well
- (NSArray *)allToManyRelationshipKeys;
    // like toManyRelationshipKeys, but returns non-class properties as well
- (NSArray *)classPropertyKeys;
    // union of toOneRelationshipKeys, toManyRelationshipKeys, and attributeKeys

@end

@interface NSArray(_EOSnapshotStuff)
- (NSMutableArray *)_mutableCopyFromSnapshot;
@end

@interface NSArray(_EOKeyValueCodingExtensions)
+ (NSArray *)aggregateKeys;
@end


@interface NSBundle (_EOLicensing)
- (BOOL)_isLicensedForFeature:(NSString *)featureName;
    // checks in License.table in bundle for key.  Returns YES if file is
    // found and key is set to YES; NO otherwise.

- (void)_raiseIfNotLicencedForFeature:(NSString *)featureName 
                             message:(NSString *)message;
    // if _isLicensedForFeature: returns NO, logs error message and raises exception.
@end


@interface EOKeyGlobalID (_EOPrivate)
- (BOOL)isFinal; // YES
@end

@interface EOKeySubclassGlobalID : EOKeyGlobalID
{
    NSString *_bestEntityName;
    NSString *_subEntityName;
}
+ (id)globalIDWithEntityName:(NSString *)entityName subEntityName:(NSString *)subEntityName bestEntityName:(NSString *)bestEntityName keys:(id *)keys keyCount:(unsigned)count
                        zone:(NSZone *)zone;
- (BOOL)isFinal; // NO if the subclass is nil
@end

@interface NSException (_EOValidationAddOns)
- (NSException *)exceptionRememberingObject:(id)eo key:(NSString *)key;
    // exception using exceptionAddingEntriesToUserInfo: to add
    // EOValidatedObjectUserInfoKey, EOValidatedPropertyUserInfoKey;
    // If either arg is nil, it is not added.
@end

@interface EOClassDescription (_EOClassDescAdditions)
- (EOClassDescription *)classDescriptionForKeyPath:(NSString *)keyPath;
@end

@interface NSObject (_EOClassDescriptionClassDelegate)
- (EOClassDescription *)classDescriptionForEntityName:(NSString *)entityName;
    // back door for WOB to determine the class description for unarchived
    // EODetailDisplayGroups (by looking up the entity in the defaultModelGroup)
@end
