/*
 * NSJavaVirtualMachine.h
 *
 * Copyright (c) 1997, Apple Computer, Inc.
 * All Rights Reserved.
 */

#import <Foundation/Foundation.h>

@interface NSJavaVirtualMachine : NSObject
{
    void *_vmdata;
    void *_vtable;
}

+ (id) defaultVirtualMachine;

+ (NSString *) defaultClassPath;

- init;
- initWithClassPath:(NSString *)classPath;

- (Class) findClass:(NSString *)javaClassName;

@end

@interface NSJavaVirtualMachine (WebObjectsPatch)

/* These arguments will be used to configure the virtual machine when it is
 * first created.  The arguments must be set before the virtual machine is
 * created; they will have no effect on a running virtual machine.  The key
 * names correspond to the names of the fields in the JDK1_1InitArgs structure
 * (defined in JDK/include/jni.h), used by the bridge to create the virtual
 * machine in the JNI_CreateJavaVM() function.  A synoposis of the valid
 * key/values (and the corresponding argument to the command-line java
 * application) follows:
 *
 *	NSJavaCheckSource	"YES" or "NO"	-cs -checksource
 *	NSJavaNativeStackSize	NSNumber	-ss<number>
 *	NSJavaStackSize		NSNumber	-oss<number>
 *	NSJavaMinHeapSize	NSNumber	-ms<number>
 *	NSJavaMaxHeapSize	NSNumber	-mx<number>
 *	NSJavaVerifyMode	"YES" or "NO"	-verify
 *	NSJavaEnableClassGC	"YES" or "NO"	NO => -noclassgc
 *	NSJavaEnableVerboseGC	"YES" or "NO"	-verbosegc
 *	NSJavaDisableAsyncGC	"YES" or "NO"	-noasyncgc
 *	NSJavaVerbose		"YES" or "NO"	-v -verbose
 *	NSJavaDebugging		"YES" or "NO"	-debug
 *	NSJavaDebugPort		NSNumber	huh???
 *
 * Please note that this mechanism of specifying arguments to the virtual
 * machine is valid for the WOF 3.5 patch only.  We reserve the right and
 * stress the probability that this may change for WOF 4.0/Rhapsody and
 * other future incarnations of this project.
 */
+ (void) setDefaultVirtualMachineArguments:(NSDictionary *)arguments;

@end


@interface NSObject (InstantiatingJavaObjects)

/* Instantiating java objects for when no -init/constructor mapping works.  The class these methods are invoked on *must* be a class returned by the -findClass: method, otherwise an NSInvalidJavaClassException. */
+ (id) newWithSignature:(NSString *)signature, ...;
+ (id) newWithSignature:(NSString *)signature arguments:(va_list)args;

@end

extern NSString *NSInvalidJavaClassException;
