#ifndef JOBJC_CLASS_H
#define JOBJC_CLASS_H

#import <ObjCJava/java-vm.h>

/*
 * Create a java object of the specified class to wrap around an Objc object.
 * Not for the faint of heart.  See comments in NSCreateJavaWrapper() for details.
 * (It just looks up the java class corresponding to the objc object and then
 * calls this function.)
 */
JOBJC_EXPORT JAVAHandle NSCreateExplicitJavaWrapper (id objcObject, JAVAClass javaClass);

/*
 * Exported for use in EOF.  Replaces the retain/release methods on a class
 * with variants that ensures proper synchronization with the asynchronous
 * garbage collector.  *Not* for public consumption!
 */
JOBJC_EXPORT void ReplaceRetainReleaseMethods (Class cls);

/*
 * Mapping between Java and Objective C classes.  If no explicit mapping has been
 * specified (using the NSSetJavaClassForObjcClass() function), a default mapping
 * is used.
 *
 * When mapping Objective C classes to Java classes, this default behaviour
 * is to search up the superclass hierarchy looking for the closest mapping
 * possible.  In all cases, this search will end with the NSObject ->
 * next.util.NextObject mapping.
 *
 * When mapping Java classes to Objective C classes, the default behaviour when
 * no explicit mapping can be found is to dynamically create an Objective C
 * class hierarchy that duplicates the Java class hierarchy.  (Of course, if the
 * shouldCreate argument is false, nil will be returned -- note that the Java
 * superclass hierarchy is *not* searched for the closest mapping.)
 */
JOBJC_EXPORT JAVAClass NSObjcClassToJavaClass (Class objcClass);
JOBJC_EXPORT Class NSJavaClassToObjcClass (JAVAClass javaClass, BOOL shouldCreate);

#endif JOBJC_CLASS_H
