/*
 * Copyright (c) 1996, NeXT Software, Inc.
 * All Rights Reserved.
 */

#ifndef JOBJC_VM_H
#define JOBJC_VM_H

#import <Foundation/Foundation.h>
#import <objc/objc-runtime.h>
#import <ObjCJava/java-objc.h>

/* See compiler-bug.s for details */
#if defined(WIN32)
JOBJC_EXPORT id objc_msgSendSuperStruct (struct objc_super *r, SEL cmd, ...);
JOBJC_EXPORT id objc_msgSendStruct (id receiver, SEL cmd, ...);
#endif


#if defined(WIN32)
    #define NSJAVA_STUB_EXPORT __declspec(dllexport)
    #define NSJAVA_STUB_IMPORT __declspec(dllimport)
#else
    #define NSJAVA_STUB_EXPORT
    #define NSJAVA_STUB_IMPORT extern
#endif


/*
 * Find out the real Objc (ie. non-generated) class.
 */
#define CLS_JAVA_WRAPPED 0x800L // XXX should put in objc-class.h
#define CLS_JAVA_CLASS 0x400L		// XXX should put in objc-class.h
#define CLS_JAVA_HYBRID 0x200L	// XXX should put in objc-class.h

static inline Class
__NSRealObjcClassForClass (Class cls)
{
    /*
     * Don't need to check validity of cls since everything will inherit
     * from NSObject or NSProxy.  If the class doesn't, you've got far
     * bigger problems to worry about.
     */
    if (cls) {
        while (cls->info & CLS_JAVA_HYBRID)
            cls = cls->super_class;
    }
    return cls;
}

static inline Class
__NSRealObjcClassForId (id obj)
{
    if (obj) return __NSRealObjcClassForClass(obj->isa);
    else return nil;
}

JOBJC_EXPORT id _NSAllocateObjcObjectForJavaHandle (JAVAHandle javaObject);
JOBJC_EXPORT void _NSAssociateJavaObject (JAVAHandle javaObject, id objcObject);
JOBJC_EXPORT void _NSAssociateJavaObjectNoRetain(JAVAHandle javaObject, id objcObject);
JOBJC_EXPORT void _NSDisassociateJavaObjectNoRelease (JAVAHandle javaObject, id objcObject);
JOBJC_EXPORT void _NSRaiseExceptionInJava (JAVAEnv env, NSException *e);
JOBJC_EXPORT void *_NSJavaStubEnter (void);
JOBJC_EXPORT void _NSJavaStubExit (void *tag);


/****************************   Apple's runtime   ****************************/

#if defined(NSJAVA_VM_APPLE)

/*
 * C representation of the java id class.  (Make sure the instance variables
 * match the class description.)
 */
typedef struct ClassId {
    id instance;
} ClassId;
HandleTo(Id);

#define NSJVM_STUB_NAME_SIMPLE(class, name, type) \
    NSJAVA_STUB_EXPORT stack_item * \
    Java_ ##class ##_ ##name ##_stub (stack_item *stack, struct execenv *ee)

#define NSJVM_STUB_NAME(package, class, name, type) \
    NSJAVA_STUB_EXPORT stack_item * \
    Java_ ##package ##_ ##class ##_ ##name ##_stub (stack_item *stack, struct execenv *ee)

#define NSJVM_STUB_ARGS_START_INSTANCE
#define NSJVM_STUB_ARGS_START_CLASS
#define NSJVM_STUB_ARG(index, type)
#define NSJVM_STUB_ARGS_END

#define NSJVM_DECLARE_CHAR(index)
#define NSJVM_DECLARE_SHORT(index)
#define NSJVM_DECLARE_INT(index)
#define NSJVM_DECLARE_LONG(index)
#define NSJVM_DECLARE_FLOAT(index)
#define NSJVM_DECLARE_DOUBLE(index) Java8 larg ##index;
#define NSJVM_DECLARE_LONGLONG(index) Java8 larg ##index;
#define NSJVM_DECLARE_ID(index)
#define NSJVM_DECLARE_HANDLE(index)

#define NSJVM_ARG(index) stack[index].p
#define NSJVM_ARG_CHAR(index) (char) stack[index].i
#define NSJVM_ARG_SHORT(index) (short) stack[index].i
#define NSJVM_ARG_INT(index) (int) stack[index].i
#define NSJVM_ARG_LONG(index) (long) stack[index].i
#define NSJVM_ARG_FLOAT(index) (float) stack[index].f
#define NSJVM_ARG_DOUBLE(index) GET_DOUBLE(larg ##index, stack+index)
#define NSJVM_ARG_LONGLONG(index) GET_INT64(larg ##index, stack+index)
#define NSJVM_ARG_ID(index) NSJavaHandleToId(stack[index].p)
#define NSJVM_ARG_HANDLE(index) ((JAVAHandle)stack[index].h)

#define NSJVM_INVOKE_VOID(value) do { value; } while (0)
#define NSJVM_INVOKE_CHAR(value) do { stack[0].i = (int) value; } while (0)
#define NSJVM_INVOKE_SHORT(value) do { stack[0].i = (int) value; } while (0)
#define NSJVM_INVOKE_INT(value) do { stack[0].i = (int) value; } while (0)
#define NSJVM_INVOKE_LONG(value) do { stack[0].i = (long) value; } while (0)
#define NSJVM_INVOKE_LONGLONG(value) do { Java8 retval; SET_INT64(retval, stack, value); } while (0)
#define NSJVM_INVOKE_ID(value) do { stack[0].p = NSIdToJavaHandle((id)(value)); } while (0)
#define NSJVM_INVOKE_FLOAT(value) do { stack[0].f = (float) value; } while (0)
#define NSJVM_INVOKE_DOUBLE(value) do { Java8 retval; SET_DOUBLE(retval, stack, value); } while (0)
#define NSJVM_INVOKE_CONVERT(value, convert) do { stack[0].p = (void *) convert(value); } while (0)
#define NSJVM_INVOKE_HANDLE(value) do { stack[0].h = value; } while (0)

#define NSJVM_RETVAL_VOID
#define NSJVM_RETVAL_CHAR
#define NSJVM_RETVAL_SHORT
#define NSJVM_RETVAL_INT
#define NSJVM_RETVAL_LONG
#define NSJVM_RETVAL_LONGLONG
#define NSJVM_RETVAL_ID
#define NSJVM_RETVAL_FLOAT
#define NSJVM_RETVAL_DOUBLE
#define NSJVM_RETVAL_HANDLE

#define NSJVM_RETTYPE_VOID return stack
#define NSJVM_RETTYPE_CHAR return stack + 1
#define NSJVM_RETTYPE_SHORT return stack + 1
#define NSJVM_RETTYPE_INT return stack + 1
#define NSJVM_RETTYPE_LONG return stack + 1
#define NSJVM_RETTYPE_LONGLONG return stack + 2
#define NSJVM_RETTYPE_ID return stack + 1
#define NSJVM_RETTYPE_FLOAT return stack + 1
#define NSJVM_RETTYPE_DOUBLE return stack + 2
#define NSJVM_RETTYPE_HANDLE return stack + 1

#define NSJVM_ENV ee
#define NSJVM_RECEIVER unhand((HId *) stack[0].p)->instance
#define NSJVM_JAVA_RECEIVER NSJVM_ARG_HANDLE(0)

#define NSJVM_STUB_BODY(locals, call, retvaldecl, rettype)		\
{									\
    void *tag;								\
    retvaldecl								\
									\
    tag = _NSJavaStubEnter();						\
									\
    NS_DURING								\
	locals								\
	call;								\
    NS_HANDLER								\
        _NSRaiseExceptionInJava(NSJVM_ENV, localException);		\
    NS_ENDHANDLER							\
									\
    _NSJavaStubExit(tag);						\
    rettype;								\
}

#define NSJVM_STUB_INIT(locals, call)					\
{									\
    JAVAHandle javaObject = (JAVAHandle) NSJVM_JAVA_RECEIVER;		\
    void *tag;								\
                                                                        \
    tag = _NSJavaStubEnter();						\
									\
    NS_DURING								\
	struct objc_super r;						\
	id objcObject;							\
	locals								\
									\
	r.receiver = _NSAllocateObjcObjectForJavaHandle(javaObject);	\
	r.class = __NSRealObjcClassForId(r.receiver);			\
        _NSAssociateJavaObjectNoRetain(javaObject, r.receiver);		\
        objcObject = call;						\
									\
        if (objcObject == nil) {					\
            JAVAThrow(NSJVM_ENV, "java/lang/NullPointerException",	\
			"alloc or init returned nil");			\
            _NSDisassociateJavaObjectNoRelease(javaObject, r.receiver);	\
        } else if (r.receiver != objcObject) {				\
            _NSDisassociateJavaObjectNoRelease(javaObject, r.receiver);	\
            _NSAssociateJavaObjectNoRetain(javaObject, objcObject);	\
	}								\
    NS_HANDLER								\
        _NSRaiseExceptionInJava(ee, localException);			\
    NS_ENDHANDLER							\
									\
    _NSJavaStubExit(tag);						\
    NSJVM_RETTYPE_VOID;							\
}

#endif


/****************************   Sun's runtime   ****************************/

#if defined(NSJAVA_VM_SUN)

/*
 * C representation of the java id class.  (Make sure the instance variables
 * match the class description.)
 */
typedef struct ClassId {
    id instance;
} ClassId;
HandleTo(Id);

#define NSJVM_STUB_NAME_SIMPLE(class, name, type) \
    NSJAVA_STUB_EXPORT stack_item * \
    Java_ ##class ##_ ##name ##_stub (stack_item *stack, struct execenv *ee)

#define NSJVM_STUB_NAME(package, class, name, type) \
    NSJAVA_STUB_EXPORT stack_item * \
    Java_ ##package ##_ ##class ##_ ##name ##_stub (stack_item *stack, struct execenv *ee)

#define NSJVM_STUB_ARGS_START_INSTANCE
#define NSJVM_STUB_ARGS_START_CLASS
#define NSJVM_STUB_ARG(index, type)
#define NSJVM_STUB_ARGS_END

#define NSJVM_DECLARE_CHAR(index)
#define NSJVM_DECLARE_SHORT(index)
#define NSJVM_DECLARE_INT(index)
#define NSJVM_DECLARE_LONG(index)
#define NSJVM_DECLARE_FLOAT(index)
#define NSJVM_DECLARE_DOUBLE(index) Java8 larg ##index;
#define NSJVM_DECLARE_LONGLONG(index) Java8 larg ##index;
#define NSJVM_DECLARE_ID(index)
#define NSJVM_DECLARE_HANDLE(index)

#define NSJVM_ARG(index) stack[index].p
#define NSJVM_ARG_CHAR(index) (char) stack[index].i
#define NSJVM_ARG_SHORT(index) (short) stack[index].i
#define NSJVM_ARG_INT(index) (int) stack[index].i
#define NSJVM_ARG_LONG(index) (long) stack[index].i
#define NSJVM_ARG_FLOAT(index) (float) stack[index].f
#define NSJVM_ARG_DOUBLE(index) GET_DOUBLE(larg ##index, stack+index)
#define NSJVM_ARG_LONGLONG(index) GET_INT64(larg ##index, stack+index)
#define NSJVM_ARG_ID(index) NSJavaHandleToId(stack[index].p)
#define NSJVM_ARG_HANDLE(index) ((JAVAHandle)stack[index].h)

#define NSJVM_INVOKE_VOID(value) do { value; } while (0)
#define NSJVM_INVOKE_CHAR(value) do { stack[0].i = (int) value; } while (0)
#define NSJVM_INVOKE_SHORT(value) do { stack[0].i = (int) value; } while (0)
#define NSJVM_INVOKE_INT(value) do { stack[0].i = (int) value; } while (0)
#define NSJVM_INVOKE_LONG(value) do { stack[0].i = (long) value; } while (0)
#define NSJVM_INVOKE_LONGLONG(value) do { Java8 retval; SET_INT64(retval, stack, value); } while (0)
#define NSJVM_INVOKE_ID(value) do { stack[0].p = NSIdToJavaHandle((id)(value)); } while (0)
#define NSJVM_INVOKE_FLOAT(value) do { stack[0].f = (float) value; } while (0)
#define NSJVM_INVOKE_DOUBLE(value) do { Java8 retval; SET_DOUBLE(retval, stack, value); } while (0)
#define NSJVM_INVOKE_CONVERT(value, convert) do { stack[0].p = (void *) convert(value); } while (0)
#define NSJVM_INVOKE_HANDLE(value) do { stack[0].h = value; } while (0)

#define NSJVM_RETVAL_VOID
#define NSJVM_RETVAL_CHAR
#define NSJVM_RETVAL_SHORT
#define NSJVM_RETVAL_INT
#define NSJVM_RETVAL_LONG
#define NSJVM_RETVAL_LONGLONG
#define NSJVM_RETVAL_ID
#define NSJVM_RETVAL_FLOAT
#define NSJVM_RETVAL_DOUBLE
#define NSJVM_RETVAL_HANDLE

#define NSJVM_RETTYPE_VOID return stack
#define NSJVM_RETTYPE_CHAR return stack + 1
#define NSJVM_RETTYPE_SHORT return stack + 1
#define NSJVM_RETTYPE_INT return stack + 1
#define NSJVM_RETTYPE_LONG return stack + 1
#define NSJVM_RETTYPE_LONGLONG return stack + 2
#define NSJVM_RETTYPE_ID return stack + 1
#define NSJVM_RETTYPE_FLOAT return stack + 1
#define NSJVM_RETTYPE_DOUBLE return stack + 2
#define NSJVM_RETTYPE_HANDLE return stack + 1

#define NSJVM_ENV ee
#define NSJVM_RECEIVER unhand((HId *) stack[0].p)->instance
#define NSJVM_JAVA_RECEIVER NSJVM_ARG_HANDLE(0)

#define NSJVM_STUB_BODY(locals, call, retvaldecl, rettype)		\
{									\
    void *tag;								\
    retvaldecl								\
									\
    tag = _NSJavaStubEnter();						\
									\
    NS_DURING								\
	locals								\
	call;								\
    NS_HANDLER								\
        _NSRaiseExceptionInJava(ee, localException);			\
    NS_ENDHANDLER							\
									\
    _NSJavaStubExit(tag);						\
    rettype;								\
}

#define NSJVM_STUB_INIT(locals, call)					\
{									\
    JAVAHandle javaObject = (JAVAHandle) NSJVM_JAVA_RECEIVER;		\
    void *tag;								\
                                                                        \
    tag = _NSJavaStubEnter();						\
									\
    NS_DURING								\
	struct objc_super r;						\
	id objcObject;							\
	locals								\
									\
	r.receiver = _NSAllocateObjcObjectForJavaHandle(javaObject);	\
	r.class = __NSRealObjcClassForId(r.receiver);			\
        _NSAssociateJavaObjectNoRetain(javaObject, r.receiver);		\
        objcObject = call;						\
									\
        if (objcObject == nil) {					\
            JAVAThrow(ee, "java/lang/NullPointerException",		\
			"alloc or init returned nil");			\
            _NSDisassociateJavaObjectNoRelease(javaObject, r.receiver);	\
        } else if (r.receiver != objcObject) {				\
            _NSDisassociateJavaObjectNoRelease(javaObject, r.receiver);	\
            _NSAssociateJavaObjectNoRetain(javaObject, objcObject);	\
	}								\
    NS_HANDLER								\
        _NSRaiseExceptionInJava(ee, localException);			\
    NS_ENDHANDLER							\
									\
    _NSJavaStubExit(tag);						\
    NSJVM_RETTYPE_VOID;							\
}

#endif


/**************************   Microsoft's runtime   **************************/


#if defined (NSJAVA_VM_JAKARTA)
#error Microsoft VM not supported.  Yet.
#endif



/**************************   Netscape's runtime   **************************/


#if defined (NSJAVA_VM_NETSCAPE)
#error Netscape VM not supported.  Yet.
#endif


#endif JOBJC_VM_H
