/*
 * Copyright (c) 1996, NeXT Software, Inc.
 * All Rights Reserved.
 */

#ifndef NSJAVA_VM_INTERFACE_H
#define NSJAVA_VM_INTERFACE_H

/**

Each of these functions must be implemented in the vm-specific header files.

typedef void *JAVAEnv;
extern JAVAEnv JAVADefaultEnv;
extern JAVAEnv JAVACurrentEnv;
extern JAVAEnv JAVAPrivilegedEnv; // may not be present in all VMs

typedef void *JAVAClass;
typedef void *JAVAHandle;
typedef void *JAVAField;
typedef void *JAVAMethod;

VM operations:

  NSJVM_EXPORT void *JAVALoadVirtualMachine (NSString *classpath);
  NSJVM_EXPORT void JAVAUnloadVirtualMachine (void *vminfo);
  NSJVM_EXPORT void JAVASetVirtualMachineArguments (NSDictionary *arguments);

Class operations:

  JAVAClass JAVAFindClass (JAVAEnv env, const char *name);
  BOOL JAVAIsSubclassOf (JAVAEnv env, JAVAClass cls, JAVAClass potentialSuper);
  BOOL JAVAIsInterface (JAVAEnv env, JAVAClass cls);
  JAVAClass JAVASuperClass (JAVAEnv env, JAVAClass cls);
  const char *JAVAClassName (JAVAEnv env, JAVAClass cls);
  BOOL JAVAClassImplementsInterface (JAVAEnv env, JAVAClass cls, JAVAClass interface);

Exceptions:

  void JAVAThrowException (JAVAEnv env, JAVAHandle exceptionObject);
  void JAVAThrow (JAVAEnv env, const char *exceptionClass, const char *message);
  JAVAHandle JAVAExceptionThatOccurred (JAVAEnv env);
  void JAVAExceptionClear (JAVAEnv env);

Object operations:

  JAVAHandle JAVANewObject (JAVAEnv env, JAVAClass class, const char *signature, BOOL shortFloats, va_list args);
  JAVAClass JAVAClassOfObject (JAVAEnv env, JAVAHandle object);
  BOOL JAVAIsInstanceOf (JAVAEnv env, JAVAHandle object, JAVAClass cls);

Field operations:

  unsigned JAVAFieldCount (JAVAEnv env, JAVAClass cls);
  JAVAField *JAVAFields (JAVAEnv env, JAVAClass cls);
  JAVAField JAVAFieldLookup (JAVAEnv env, JAVAClass cls, const char *name, const char *signature);
  const char *JAVAFieldSignature (JAVAEnv env, JAVAField field);

  JAVAHandle JAVAFieldGet (JAVAEnv env, JAVAField field, JAVAHandle object);
  int JAVAFieldGetInt (JAVAEnv env, JAVAField field, JAVAHandle object);
  long long JAVAFieldGetInt64 (JAVAEnv env, JAVAField field, JAVAHandle object);
  float JAVAFieldGetFloat (JAVAEnv env, JAVAField field, JAVAHandle object);
  double JAVAFieldGetDouble (JAVAEnv env, JAVAField field, JAVAHandle object);

  void JAVAFieldSet (JAVAEnv env, JAVAField field, JAVAHandle object, JAVAHandle value);
  void JAVAFieldSetInt (JAVAEnv env, JAVAField field, JAVAHandle object, int value);
  void JAVAFieldSetInt64 (JAVAEnv env, JAVAField field, JAVAHandle object, long long value);
  void JAVAFieldSetFloat (JAVAEnv env, JAVAField field, JAVAHandle object, float value);
  void JAVAFieldSetDouble (JAVAEnv env, JAVAField field, JAVAHandle object, double value);

Method operations:

  unsigned JAVAMethodCount (JAVAEnv env, JAVAClass cls);
  JAVAMethod JAVAMethodAtIndex(JAVAEnv env, JAVAClass cls, int index);
  BOOL JAVAMethodIsClassMethod (JAVAEnv env, JAVAMethod method);
  BOOL JAVAMethodIsConstructor (JAVAEnv env, JAVAMethod method);
  JAVAMethod JAVAMethodLookup (JAVAEnv env, JAVAClass cls, const char *name, const char *signature, BOOL searchSuperClass); // signature may be NULL in which case the first method with a matching name is returned.  If searchSuperClass is false, only methods explicitly declared in the specified class will be checked.
  const char *JAVAMethodName (JAVAEnv env, JAVAMethod method);
  const char *JAVAMethodSignature (JAVAEnv env, JAVAMethod method);
  JAVAHandle JAVAMethodInvoke (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
  
  Convenience functions have the form:
    <type> JAVAMethodInvoke<Type> (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);

    void JAVAMethodInvokeVoid (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    BOOL JAVAMethodInvokeBoolean (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    char JAVAMethodInvokeByte (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    short JAVAMethodInvokeChar (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    short JAVAMethodInvokeShort (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    int JAVAMethodInvokeInt (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    long long JAVAMethodInvokeLong (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    float JAVAMethodInvokeFloat (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    double JAVAMethodInvokeDouble (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);

Static method operations:

  JAVAMethod JAVAStaticMethodLookup (JAVAEnv env, JAVAClass cls, const char *name, const char *signature);
  JAVAHandle JAVAStaticMethodInvoke (JAVAEnv env, JAVAClass target, JAVAMethod method, BOOL shortFloats, va_list args);
  
  Convenience functions have the form:
    <type> JAVAStaticMethodInvoke<Type> (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);

    void JAVAStaticMethodInvokeVoid (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    BOOL JAVAStaticMethodInvokeBoolean (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    char JAVAStaticMethodInvokeByte (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    short JAVAStaticMethodInvokeChar (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    short JAVAStaticMethodInvokeShort (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    int JAVAStaticMethodInvokeInt (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    long long JAVAStaticMethodInvokeLong (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    float JAVAStaticMethodInvokeFloat (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);
    double JAVAStaticMethodInvokeDouble (JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL shortFloats, va_list args);

String operations:

  JAVAHandle JAVANewString (JAVAEnv env, short *unicharBuffer, unsigned count);
  unsigned JAVAStringLength (JAVAEnv env, JAVAHandle string);
  void JAVAStringChars (JAVAEnv env, JAVAHandle string, short *unicharBuffer, unsigned bufsizeInBytes);
  JAVAHandle JAVANewCString (JAVAEnv env, char *charBuffer, unsigned count);
  unsigned JAVACStringLength (JAVAEnv env, JAVAHandle string);
  void JAVACStringChars (JAVAEnv env, JAVAHandle string, char *charBuffer, unsigned bufsize);

Array operations:

  JAVAHandle JAVANewScalarArray (JAVAEnv env, unsigned length, char *signature);
  unsigned JAVAScalarArrayLength (JAVAEnv env, JAVAHandle primitiveArray);
  void *JAVAScalarArrayElements (JAVAEnv env, JAVAHandle primitiveArray);
  JAVAHandle JAVANewObjectArray (JAVAEnv env, unsigned length, JAVAClass cls);
  unsigned JAVAObjectArrayLength (JAVAEnv env, JAVAHandle objectArray);
  JAVAHandle JAVAObjectArrayGetElement (JAVAEnv env, JAVAHandle objectArray, unsigned index);
  void JAVAObjectArraySetElement (JAVAEnv env, JAVAHandle objectArray, unsigned index, JAVAHandle value);

**/


/* Pick a default VM to build for. */
#if !defined(NSJAVA_VM_APPLE) && !defined(NSJAVA_VM_SUN) && !defined(NSJAVA_VM_JAKARTA) && !defined(NSJAVA_VM_NETSCAPE)
    #if defined(NeXT)
        #define NSJAVA_VM_APPLE
    #else
        #define NSJAVA_VM_SUN
    #endif
#endif


/*
 * Need special things for DLLs on Windows.
 */
#if defined(WIN32)
    #if defined(NSBUILDINGJOBJC)
        #define NSJVM_EXPORT __declspec(dllexport)
    #else
        #define NSJVM_EXPORT __declspec(dllimport)
    #endif
#else
    #define NSJVM_EXPORT extern
#endif


/*
 * Usefull for constructing signature arguments:
 *
 * NSJSIG_METHOD( NSJSIG_CLASS("com/apple/core/Array") NSJSIG_ARRAY(NSJSIG_INT) )
 *
 */
#define NSJSIG_ARRAY(type) "[" type
#define NSJSIG_CLASS(type) "L" type ";"
#define NSJSIG_METHOD(args) "(" args ")"

#define NSJSIG_BOOLEAN "Z"
#define NSJSIG_VOID "V"
#define NSJSIG_BYTE "B"
#define NSJSIG_CHAR "C"
#define NSJSIG_SHORT "S"
#define NSJSIG_INT "I"
#define NSJSIG_LONG "J"
#define NSJSIG_FLOAT "F"
#define NSJSIG_DOUBLE "D"

#define NSJSIGCHAR_BOOLEAN 'Z'
#define NSJSIGCHAR_VOID 'V'
#define NSJSIGCHAR_BYTE 'B'
#define NSJSIGCHAR_CHAR 'C'
#define NSJSIGCHAR_SHORT 'S'
#define NSJSIGCHAR_INT 'I'
#define NSJSIGCHAR_LONG 'J'
#define NSJSIGCHAR_FLOAT 'F'
#define NSJSIGCHAR_DOUBLE 'D'
#define NSJSIGCHAR_CLASS 'L'

#if defined(NSJAVA_VM_APPLE)
#import <ObjCJava/vm-apple.h>
#elif defined(NSJAVA_VM_SUN)
#import <ObjCJava/vm-sun.h>
#elif defined(NSJAVA_VM_JAKARTA)
#import <ObjCJava/vm-jakarta.h>
#elif defined(NSJAVA_VM_NETSCAPE)
#import <ObjCJava/vm-netscape.h>
#else
#error No VM specified
#endif


/*
 * Some convenience methods defined in terms of the above primitives.
 */
#define JAVAClassNameOfObject(env, object) JAVAClassName((env), JAVAClassOfObject((env), (object)))

#endif NSJAVA_VM_INTERFACE_H
