/*
 * Copyright (c) 1996, NeXT Software, Inc.
 * All Rights Reserved.
 */

#ifndef NSJAVA_SUN_INTERFACE_H
#define NSJAVA_SUN_INTERFACE_H

#if defined (WIN32)
#define __int64 long long
#endif

#ifdef __svr4__
#define HAVE_ALIGNED_LONGLONGS 1
#define HAVE_ALIGNED_DOUBLES 1
#endif

#if !defined(__BRIDGET__)
#import <StubPreamble.h>
#import <javaString.h>

/* #define'ing trace in interpreter.h could be considered anti-social behaviour.  We attempt to ameliorate this situation. */
#undef trace
#undef tracem
#else
typedef void *ExecEnv;
typedef void *ClassClass;
typedef void *HObject;
#endif


typedef ExecEnv *JAVAEnv;
#define JAVADefaultEnv NULL	// There is a DefaultExecEnv variable, but it's not exported on Windows
#define JAVACurrentEnv EE()
#define JAVAPrivilegedEnv PRIVILEGED_EE



typedef ClassClass *JAVAClass;
typedef HObject *JAVAHandle;
typedef struct fieldblock *JAVAField;
typedef struct methodblock *JAVAMethod;


/*
 * Virtual Machine loading
 */
NSJVM_EXPORT void JAVASetVirtualMachineArguments (NSDictionary *arguments);
NSJVM_EXPORT void *JAVALoadVirtualMachine (NSString *classpath);
NSJVM_EXPORT void JAVAUnloadVirtualMachine (void *vminfo);
NSJVM_EXPORT JAVAClass JAVAAddClassFile (void *vminfo, const char *name, const void *classFileData, unsigned dataLength);

/*
 * Class operations
 */
static inline JAVAClass
JAVAFindClass (JAVAEnv env, char *name)
{
    if (NULL == name) {
        [NSException raise:NSInternalInconsistencyException format:@"JAVAFindClass() called with a NULL java class name"];
    }
    return FindClass(env, name, TRUE);
}
//#define JAVAFindClass(env, name) FindClass(env, (char *)name, TRUE)
NSJVM_EXPORT BOOL JAVAIsSubclassOf (JAVAEnv env, JAVAClass cls, JAVAClass superClass);
#define JAVAIsInterface(env, cls) cbIsInterface(cls)
#define JAVASuperClass(env, cls) cbSuperclass(cls)
#define JAVAClassName(env, cls) cbName(cls)
#define JAVAClassImplementsInterface(env, cls, interface) ImplementsInterface(cls, interface, env)


/*
 * Exceptions
 */
// Not valid with JAVADefaultEnv or JAVAPrivilegedEnv
#define JAVAThrowException(env, exceptionObject) exceptionThrow(env, exceptionObject)
#define JAVAThrow(env, clsName, message) SignalError(env, clsName, message)
#define JAVAExceptionThatOccurred(env) (exceptionOccurred(env) ? (env)->exception.exc : NULL)
#define JAVAExceptionClear(env) exceptionClear(env)


/*
 * Operations on objects
 */
NSJVM_EXPORT JAVAHandle JAVANewObject (JAVAEnv env, JAVAClass class, const char *signature, int shortFloats, va_list args);
// XXX #define get_classblock(o) ((obj_classblock(o) == get_classClass()) ? \
//                           (ClassClass *)unhand(o) : obj_classblock(o))
#define JAVAClassOfObject(env, object) obj_classblock((JHandle *)(object))
#define JAVAIsInstanceOf(env, object, cls) is_instance_of(object, cls, env)

/*
 * Field operations
 */
NSJVM_EXPORT JAVAField JAVAFieldLookup (JAVAEnv env, JAVAClass cls, const char *name, const char *signature /* may be NULL */);
#define JAVAFieldSignature(env, field) fieldsig(field)
#define JAVAFieldGet(env, field, object) (*(HObject **)((char *)unhand(object) + (field)->u.offset))
#define JAVAFieldGetInt(env, field, object) (*(int *)((char *)unhand(object) + (field)->u.offset))
#define JAVAFieldGetInt64(env, field, object) (*(long long *)((char *)unhand(object) + (field)->u.offset))
#define JAVAFieldGetFloat(env, field, object) (*(float *)((char *)unhand(object) + (field)->u.offset))
#define JAVAFieldGetDouble(env, field, object) (*(double *)((char *)unhand(object) + (field)->u.offset))
#define JAVAFieldSet(env, field, object, value) ((*(HObject **)((char *)unhand(object) + (field)->u.offset)) = value)
#define JAVAFieldSetInt(env, field, object, value) ((*(int *)((char *)unhand(object) + (field)->u.offset)) = value)
#define JAVAFieldSetInt64(env, field, object, value) ((*(long long *)((char *)unhand(object) + (field)->u.offset)) = value)
#define JAVAFieldSetFloat(env, field, object, value) ((*(float *)((char *)unhand(object) + (field)->u.offset)) = value)
#define JAVAFieldSetDouble(env, field, object, value) ((*(double *)((char *)unhand(object) + (field)->u.offset)) = value)

/*
 * Method operations
 */
NSJVM_EXPORT long __JAVAMethodInvoke(JAVAEnv env, JAVAHandle target, JAVAMethod method, BOOL isStatic, BOOL shortFloats, va_list args, long *hibits);

#define JAVAMethodCount(env, cls) cbMethodsCount(cls)
#define JAVAMethodAtIndex(env, cls, index) (cbMethods(cls) + (index))
NSJVM_EXPORT JAVAMethod JAVAConstructorLookup (JAVAEnv env, JAVAClass cls, const char *signature);
NSJVM_EXPORT JAVAMethod JAVAMethodLookup (JAVAEnv env, JAVAClass cls, const char *name, const char *signature, int searchSuperClass);
#define JAVAMethodName(env, method) ((method)->fb.name)
#define JAVAMethodSignature(env, method) ((method)->fb.signature)
#define JAVAMethodIsClassMethod(env, method) (method->fb.access & ACC_STATIC)
#define JAVAMethodIsConstructor(env, method) ('<' == *JAVAMethodName(env, method))
#define JAVAMethodInvoke(env, target, method, shortFloats, args) ((JAVAHandle) __JAVAMethodInvoke(env, target, method, FALSE, shortFloats, args, NULL))
#define JAVAMethodInvokeBoolean(env, target, method, shortFloats, args) ((char) __JAVAMethodInvoke(env, target, method, FALSE, shortFloats, args, NULL))
#define JAVAMethodInvokeByte(env, target, method, shortFloats, args) ((char) __JAVAMethodInvoke(env, target, method, FALSE, shortFloats, args, NULL))
#define JAVAMethodInvokeChar(env, target, method, shortFloats, args) ((short) __JAVAMethodInvoke(env, target, method, FALSE, shortFloats, args, NULL))
#define JAVAMethodInvokeShort(env, target, method, shortFloats, args) ((short) __JAVAMethodInvoke(env, target, method, FALSE, shortFloats, args, NULL))
#define JAVAMethodInvokeInt(env, target, method, shortFloats, args) ((int) __JAVAMethodInvoke(env, target, method, FALSE, shortFloats, args, NULL))
static inline long long
JAVAMethodInvokeLong(JAVAEnv env, JAVAHandle target, JAVAMethod method, int shortFloats, va_list args)
{
    long lobits;
    long hibits;
    
    lobits = __JAVAMethodInvoke(env, target, method, FALSE, shortFloats, args, &hibits);
#ifdef __BIG_ENDIAN__
    return (long long)hibits << 32 | lobits;
#else
    return (long long)lobits << 32 | hibits;
#endif
}
static inline float
JAVAMethodInvokeFloat (JAVAEnv env, JAVAHandle target, JAVAMethod method, int shortFloats, va_list args)
{
    union {
	long l;
	float f;
    } retval;
    retval.l = __JAVAMethodInvoke(env, target, method, FALSE, shortFloats, args, NULL);
    return retval.f;
}
static inline double
JAVAMethodInvokeDouble (JAVAEnv env, JAVAHandle target, JAVAMethod method, int shortFloats, va_list args)
{
    union {
	double d;
        struct { long hi; long lo; } l;
    } retval;

    retval.l.lo = __JAVAMethodInvoke(env, target, method, FALSE, shortFloats, args, &retval.l.hi);
    return retval.d;
}



#define JAVAStaticMethodLookup(env, cls, name, signature) __not_implemented__
#define JAVAStaticMethodInvoke(env, target, method, shortFloats, args) ((JAVAHandle) __JAVAMethodInvoke(env, (JAVAHandle)(target), method, TRUE, shortFloats, args, NULL))
#define JAVAStaticMethodInvokeBoolean(env, target, method, shortFloats, args) ((int) __JAVAMethodInvoke(env, (JAVAHandle)(target), method, TRUE, shortFloats, args, NULL))
#define JAVAStaticMethodInvokeByte(env, target, method, shortFloats, args) ((char) __JAVAMethodInvoke(env, (JAVAHandle)(target), method, TRUE, shortFloats, args, NULL))
#define JAVAStaticMethodInvokeChar(env, target, method, shortFloats, args) ((short) __JAVAMethodInvoke(env, (JAVAHandle)(target), method, TRUE, shortFloats, args, NULL))
#define JAVAStaticMethodInvokeShort(env, target, method, shortFloats, args) ((short) __JAVAMethodInvoke(env, (JAVAHandle)(target), method, TRUE, shortFloats, args, NULL))
#define JAVAStaticMethodInvokeInt(env, target, method, shortFloats, args) ((int) __JAVAMethodInvoke(env, (JAVAHandle)(target), method, TRUE, shortFloats, args, NULL))
static inline long long
JAVAStaticMethodInvokeLong (JAVAEnv env, JAVAClass target, JAVAMethod method, int shortFloats, va_list args)
{
    long lobits;
    long hibits;

    lobits = __JAVAMethodInvoke(env, (JAVAHandle) target, method, TRUE, shortFloats, args, &hibits);
#ifdef __BIG_ENDIAN__
    return (long long)hibits << 32 | lobits;
#else
    return (long long)lobits << 32 | hibits;
#endif
}
static inline float
JAVAStaticMethodInvokeFloat (JAVAEnv env, JAVAClass target, JAVAMethod method, int shortFloats, va_list args)
{
    union {
	long l;
	float f;
    } retval;
    retval.l = __JAVAMethodInvoke(env, (JAVAHandle) target, method, TRUE, shortFloats, args, NULL);
    return retval.f;
}
static inline double
JAVAStaticMethodInvokeDouble (JAVAEnv env, JAVAClass target, JAVAMethod method, int shortFloats, va_list args)
{
    union {
        double d;
        struct { long hi; long lo; } l;
    } retval;

    retval.l.lo = __JAVAMethodInvoke(env, (JAVAHandle) target, method, TRUE, shortFloats, args, &retval.l.hi);
    return retval.d;
}


/*
 * String operations
 */
NSJVM_EXPORT JAVAHandle JAVANewString (JAVAEnv env, short *unicharBuffer, unsigned count);
#define JAVAStringLength(env, string) javaStringLength((Hjava_lang_String *)(string))
#define JAVAStringChars(env, string, buffer, bufsize) javaString2unicode((Hjava_lang_String *)(string), buffer, bufsize)
#define JAVANewCString(env, buffer, count) ((JAVAHandle)makeJavaString((char *)(buffer), count))
#define JAVACStringLength(env, string) javaStringLength((Hjava_lang_String *)(string))
#define JAVACStringChars(env, string, buffer, bufsize) javaString2CString((Hjava_lang_String *)(string), buffer, bufsize)


/*
 * Array operations
 */
NSJVM_EXPORT JAVAHandle JAVANewScalarArray (JAVAEnv env, unsigned length, char *signature);
#define JAVAScalarArrayLength(env, array) obj_length(array)
#define JAVAScalarArrayElements(env, array) ((void *)(unhand((HArrayOfByte *)(array))->body))
NSJVM_EXPORT JAVAHandle JAVANewObjectArray (JAVAEnv env, unsigned length, JAVAClass cls);
#define JAVAObjectArrayLength(env, array) obj_length(array)
#define JAVAObjectArrayGetElement(env, array, index) (unhand((HArrayOfObject *)(array))->body[index])
#define JAVAObjectArraySetElement(env, array, index, value) (unhand((HArrayOfObject *)(array))->body[index] = (value))

#endif NSJAVA_SUN_INTERFACE_H
